@@ -135,7 +135,7 @@ impl<'a> HyperlinkCollection<'a> {
     fn resolve_label2label_references(&mut self) {
         let mut nb_no_match = 0;
         let mut idx = 0;
-        while self.label2label.len() > 0 && nb_no_match < self.label2label.len() {
+        while !self.label2label.is_empty() && nb_no_match < self.label2label.len() {
             let (key_alias, key) = &self.label2label[idx];
             // This makes sure, that we advance in the loop.
             if let Some(value) = self.label2dest.get(key) {
@@ -402,6 +402,7 @@ impl<'a> Hyperlink<'a> {
 /// * `link_content = (link_text, link_destination, link_title)`
 ///
 impl<'a> Iterator for Hyperlink<'a> {
+    #[allow(clippy::type_complexity)]
     type Item = (
         (&'a str, &'a str, &'a str),
         (Cow<'a, str>, Cow<'a, str>, Cow<'a, str>),
@@ -434,9 +435,8 @@ impl<'a> Iterator for Hyperlink<'a> {
         let mut again = true;
         while again {
             status = match status {
-                // Cloning a pointer is cheap.
                 // Advance state machine and match one more time.
-                Status::Init => Status::DirectSearch(self.input.clone()),
+                Status::Init => Status::DirectSearch(self.input),
 
                 Status::DirectSearch(input) => {
                     // We stay in direct mode.
@@ -471,7 +471,7 @@ impl<'a> Iterator for Hyperlink<'a> {
                 }
 
                 Status::ResolvedLinks(mut resolved_links) => {
-                    while resolved_links.len() > 0 {
+                    while !resolved_links.is_empty() {
                         if let (input_offset, len, Link::Text2Dest(te, de, ti)) =
                             resolved_links.remove(0)
                         {
